library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity uplport is
  port (
    UPLGlobalClk : in std_logic;
    UPLReset : in std_logic;
    ready : out std_logic;
    done : in std_logic;
    send_length : in signed(32-1 downto 0);
    recv_length : out signed(32-1 downto 0);
    data_length : out signed(32-1 downto 0);
    data_address : in signed(32-1 downto 0);
    data_dout : out signed(32-1 downto 0);
    data_din : in signed(32-1 downto 0);
    data_we : in std_logic;
    data_oe : in std_logic;
    UPLIn_Data : in std_logic_vector(32-1 downto 0);
    UPLIn_En : in std_logic;
    UPLIn_Request : in std_logic;
    UPLIn_Ack : out std_logic;
    UPLOut_Data : out std_logic_vector(32-1 downto 0);
    UPLOut_En : out std_logic;
    UPLOut_Request : out std_logic;
    UPLOut_Ack : in std_logic
  );
end uplport;

architecture RTL of uplport is

  attribute mark_debug : string;
  attribute keep : string;

  component dualportram
    generic (
      WIDTH : integer := 32;
      DEPTH : integer := 10;
      WORDS : integer := 1024
    );
    port (
      clk : in std_logic;
      reset : in std_logic;
      length : out signed(31 downto 0);
      address : in signed(31 downto 0);
      din : in signed(WIDTH-1 downto 0);
      dout : out signed(WIDTH-1 downto 0);
      we : in std_logic;
      oe : in std_logic;
      address_b : in signed(31 downto 0);
      din_b : in signed(WIDTH-1 downto 0);
      dout_b : out signed(WIDTH-1 downto 0);
      we_b : in std_logic;
      oe_b : in std_logic
    );
  end component dualportram;

  signal UPLGlobalClk_sig : std_logic;
  signal UPLReset_sig : std_logic;
  signal ready_sig : std_logic := '0';
  signal done_sig : std_logic;
  signal send_length_sig : signed(32-1 downto 0);
  signal recv_length_sig : signed(32-1 downto 0) := (others => '0');
  signal data_length_sig : signed(32-1 downto 0);
  signal data_address_sig : signed(32-1 downto 0);
  signal data_dout_sig : signed(32-1 downto 0);
  signal data_din_sig : signed(32-1 downto 0);
  signal data_we_sig : std_logic;
  signal data_oe_sig : std_logic;
  signal UPLIn_Data_sig : std_logic_vector(32-1 downto 0);
  signal UPLIn_En_sig : std_logic;
  signal UPLIn_Request_sig : std_logic;
  signal UPLIn_Ack_sig : std_logic := '0';
  signal UPLOut_Data_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal UPLOut_En_sig : std_logic := '0';
  signal UPLOut_Request_sig : std_logic := '0';
  signal UPLOut_Ack_sig : std_logic;

  signal U_clk : std_logic;
  signal U_reset : std_logic;
  signal U_length : signed(32-1 downto 0);
  signal U_address : signed(32-1 downto 0);
  signal U_din : signed(32-1 downto 0);
  signal U_dout : signed(32-1 downto 0);
  signal U_we : std_logic;
  signal U_oe : std_logic;
  signal U_address_b : signed(32-1 downto 0) := (others => '0');
  signal U_din_b : signed(32-1 downto 0) := (others => '0');
  signal U_dout_b : signed(32-1 downto 0);
  signal U_we_b : std_logic := '0';
  signal U_oe_b : std_logic := '0';
  signal send_count : signed(32-1 downto 0) := (others => '0');
  signal recv_count : signed(32-1 downto 0) := (others => '0');
  type Type_main is (
    main_IDLE,
    main_RECV_WAIT,
    main_RECV_DATA,
    main_OPERATION,
    main_SEND_WAIT,
    main_MEM_WAIT,
    main_SEND_DATA  
  );
  signal main : Type_main := main_IDLE;
  signal main_delay : signed(32-1 downto 0) := (others => '0');
  signal tmp_0001 : std_logic;
  signal tmp_0002 : signed(32-1 downto 0);
  signal tmp_0003 : signed(32-1 downto 0);
  signal tmp_0004 : signed(32-1 downto 0);
  signal tmp_0005 : signed(32-1 downto 0);
  signal tmp_0006 : signed(32-1 downto 0);
  signal tmp_0007 : signed(32-1 downto 0);
  signal tmp_0008 : signed(32-1 downto 0);
  signal tmp_0009 : signed(32-1 downto 0);
  signal tmp_0010 : signed(32-1 downto 0);
  signal tmp_0011 : std_logic;
  signal tmp_0012 : std_logic;
  signal tmp_0013 : std_logic;
  signal tmp_0014 : std_logic;
  signal done_reg : std_logic := '0';
  signal tmp_0015 : std_logic;
  signal tmp_0016 : std_logic;
  signal tmp_0017 : std_logic;
  signal tmp_0018 : std_logic;
  signal tmp_0019 : std_logic;
  signal tmp_0020 : std_logic;

begin

  UPLGlobalClk_sig <= UPLGlobalClk;
  UPLReset_sig <= UPLReset;
  ready <= ready_sig;
  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        ready_sig <= '0';
      else
        if main = main_OPERATION then
          ready_sig <= '1';
        else
          ready_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  done_sig <= done;
  send_length_sig <= send_length;
  recv_length <= recv_length_sig;
  data_length <= data_length_sig;
  data_length_sig <= U_length;

  data_address_sig <= data_address;
  data_dout <= data_dout_sig;
  data_dout_sig <= U_dout;

  data_din_sig <= data_din;
  data_we_sig <= data_we;
  data_oe_sig <= data_oe;
  UPLIn_Data_sig <= UPLIn_Data;
  UPLIn_En_sig <= UPLIn_En;
  UPLIn_Request_sig <= UPLIn_Request;
  UPLIn_Ack <= UPLIn_Ack_sig;
  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        UPLIn_Ack_sig <= '0';
      else
        if main = main_RECV_WAIT then
          UPLIn_Ack_sig <= tmp_0001;
        else
          UPLIn_Ack_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  UPLOut_Data <= UPLOut_Data_sig;
  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        UPLOut_Data_sig <= (others => '0');
      else
        if main = main_SEND_DATA then
          UPLOut_Data_sig <= std_logic_vector(U_dout);
        end if;
      end if;
    end if;
  end process;

  UPLOut_En <= UPLOut_En_sig;
  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        UPLOut_En_sig <= '0';
      else
        if main = main_SEND_DATA then
          UPLOut_En_sig <= '1';
        else
          UPLOut_En_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  UPLOut_Request <= UPLOut_Request_sig;
  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        UPLOut_Request_sig <= '0';
      else
        if main = main_SEND_WAIT then
          UPLOut_Request_sig <= '1';
        else
          UPLOut_Request_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  UPLOut_Ack_sig <= UPLOut_Ack;

  -- expressions
  tmp_0001 <= not UPLIn_En_sig;
  tmp_0002 <= X"00000001" when UPLIn_En_sig = '1' else X"00000000";
  tmp_0003 <= U_address_b + X"00000001";
  tmp_0004 <= tmp_0003 when UPLIn_En_sig = '1' else U_address_b;
  tmp_0005 <= recv_count + X"00000001";
  tmp_0006 <= tmp_0005 when UPLIn_En_sig = '1' else recv_count;
  tmp_0007 <= send_length_sig;
  tmp_0008 <= U_address_b + X"00000001";
  tmp_0009 <= send_count - X"00000001";
  tmp_0010 <= U_address_b + X"00000001";
  tmp_0011 <= '1' when UPLIn_En_sig = '1' else '0';
  tmp_0012 <= '1' when UPLIn_En_sig = '0' else '0';
  tmp_0013 <= '1' when send_length_sig > X"00000000" else '0';
  tmp_0014 <= not tmp_0013;
  tmp_0015 <= not done_reg;
  tmp_0016 <= done_sig and tmp_0015;
  tmp_0017 <= tmp_0016 and tmp_0013;
  tmp_0018 <= tmp_0016 and tmp_0014;
  tmp_0019 <= '1' when UPLOut_Ack_sig = '1' else '0';
  tmp_0020 <= '1' when send_count = X"00000001" else '0';

  -- sequencers
  process (UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        main <= main_IDLE;
        main_delay <= (others => '0');
      else
        case (main) is
          when main_IDLE => 
            main <= main_RECV_WAIT;
          when main_RECV_WAIT => 
            if tmp_0011 = '1' then
              main <= main_RECV_DATA;
            end if;
          when main_RECV_DATA => 
            if tmp_0012 = '1' then
              main <= main_OPERATION;
            end if;
          when main_OPERATION => 
            if tmp_0017 = '1' then
              main <= main_SEND_WAIT;
            elsif tmp_0018 = '1' then
              main <= main_IDLE;
            end if;
          when main_SEND_WAIT => 
            if tmp_0019 = '1' then
              main <= main_MEM_WAIT;
            end if;
          when main_MEM_WAIT => 
            main <= main_SEND_DATA;
          when main_SEND_DATA => 
            if tmp_0020 = '1' then
              main <= main_IDLE;
            end if;
          when others => null;
        end case;
        done_reg <= done_sig;
      end if;
    end if;
  end process;


  U_clk <= UPLGlobalClk_sig;

  U_reset <= UPLReset_sig;

  U_address <= data_address_sig;

  U_din <= data_din_sig;

  U_we <= data_we_sig;

  U_oe <= data_oe_sig;

  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        U_address_b <= (others => '0');
      else
        if main = main_RECV_WAIT then
          U_address_b <= X"00000000";
        elsif main = main_RECV_DATA then
          U_address_b <= tmp_0004;
        elsif main = main_SEND_WAIT then
          U_address_b <= X"00000000";
        elsif main = main_MEM_WAIT then
          U_address_b <= tmp_0008;
        elsif main = main_SEND_DATA then
          U_address_b <= tmp_0010;
        end if;
      end if;
    end if;
  end process;

  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        send_count <= (others => '0');
      else
        if main = main_SEND_WAIT then
          send_count <= tmp_0007;
        elsif main = main_SEND_DATA then
          send_count <= tmp_0009;
        end if;
      end if;
    end if;
  end process;

  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        recv_count <= (others => '0');
      else
        if main = main_RECV_WAIT then
          recv_count <= tmp_0002;
        elsif main = main_RECV_DATA then
          recv_count <= tmp_0006;
        end if;
      end if;
    end if;
  end process;


  inst_U : dualportram
  generic map(
    WIDTH => 32,
    DEPTH => 10,
    WORDS => 1024
  )
  port map(
    clk => U_clk,
    reset => U_reset,
    length => U_length,
    address => U_address,
    din => U_din,
    dout => U_dout,
    we => U_we,
    oe => U_oe,
    address_b => U_address_b,
    din_b => U_din_b,
    dout_b => U_dout_b,
    we_b => U_we_b,
    oe_b => U_oe_b
  );


end RTL;
